package com.liviutudor;

import java.io.IOException;
import java.lang.management.ManagementFactory;

import javax.management.Attribute;
import javax.management.AttributeList;
import javax.management.AttributeNotFoundException;
import javax.management.DynamicMBean;
import javax.management.InvalidAttributeValueException;
import javax.management.MBeanAttributeInfo;
import javax.management.MBeanInfo;
import javax.management.MBeanOperationInfo;
import javax.management.MBeanServer;
import javax.management.ObjectName;
import javax.management.ReflectionException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

/**
 * Simple servlet that just displays a message that can be set via JMX.
 *
 * @author Liviu Tudor http://about.me/liviutudor
 */
public class SimpleServlet extends HttpServlet implements DynamicMBean {
    /** Class serial. */
    private static final long   serialVersionUID = 1L;

    /** Default message used. */
    private static final String DEF_MESSAGE      = "Mary had a little lamb. The doctor was very very surprised!";

    /** Current message to print out. */
    private String              message;

    /**
     * Default constructor. Simply sets {@link #message the message} to
     * {@link #DEF_MESSAGE the default message}.
     */
    public SimpleServlet() {
        this.message = DEF_MESSAGE;
        /*
         * Register with JMX
         */
        MBeanServer mbs = ManagementFactory.getPlatformMBeanServer();
        ObjectName oName;
        try {
            oName = new ObjectName("SERVLET:name=SimpleServlet");
            mbs.registerMBean(this, oName);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    /**
     * Used internally to set the necessary headers for our response.
     *
     * @param resp
     *            Servlet response to set headers for
     * @throws IOException
     *             thrown if any I/O errors occur
     */
    private void setHeaders(HttpServletResponse resp) throws IOException {
        resp.addHeader("Content-Type", "text/html");
        resp.addHeader("Expires", "-1");
        resp.addHeader("Pragma", "no-cache");
    }

    @Override
    protected void service(HttpServletRequest req, HttpServletResponse resp) throws ServletException, IOException {
        setHeaders(resp);

        String copy = null;
        synchronized (this) {
            copy = message;
        }

        StringBuilder msg = new StringBuilder("<h1>Message of the day is</h1>");
        if (DEF_MESSAGE.equals(copy)) {
            msg.append("<p>");
        } else {
            msg.append("<p style=\"color:red\">");
        }
        msg.append(copy).append("</p>");

        resp.getOutputStream().print(msg.toString());
    }

    public synchronized final String getMessage() {
        return message;
    }

    public synchronized final void setMessage(String message) {
        this.message = message;
    }

    public synchronized final void doubleMessage() {
        this.message = message + message;
    }

    /** Name of the JMX attribute. */
    private static final String ATTR_MESSAGE = "Message";

    /** Name of the JMX operation. */
    private static final String OP_DOUBLE    = "doubleMessage";

    @Override
    public Object getAttribute(String attribute) throws AttributeNotFoundException {
        if (ATTR_MESSAGE.equals(attribute)) {
            return getMessage();
        }
        throw new AttributeNotFoundException(attribute);
    }

    @Override
    public AttributeList getAttributes(String[] attributes) {
        AttributeList result = new AttributeList();
        Object o;
        for (String a : attributes) {
            try {
                o = getAttribute(a);
                result.add(new Attribute(a, o));
            } catch (AttributeNotFoundException e) {
                e.printStackTrace();
            }
        }
        return result;
    }

    @Override
    public MBeanInfo getMBeanInfo() {
        MBeanAttributeInfo[] attr = new MBeanAttributeInfo[] {new MBeanAttributeInfo(ATTR_MESSAGE,
                String.class.getName(), "The message to be displayed.", true, true, false)};
        MBeanOperationInfo[] op = new MBeanOperationInfo[] {new MBeanOperationInfo(OP_DOUBLE,
                "Doubles the current message", null, void.class.getName(), MBeanOperationInfo.ACTION)};

        return new MBeanInfo(SimpleServlet.class.getName(), "Simple servlet", attr, null, op, null);
    }

    @Override
    public Object invoke(String actionName, Object[] params, String[] signature) throws ReflectionException {
        if (OP_DOUBLE.equals(actionName)) {
            doubleMessage();
            return null;
        }
        throw new ReflectionException(new UnsupportedOperationException(actionName));
    }

    @Override
    public void setAttribute(Attribute attribute) throws AttributeNotFoundException,
            InvalidAttributeValueException {
        if (ATTR_MESSAGE.equals(attribute.getName())) {
            if (!(attribute.getValue() instanceof String)) {
                throw new InvalidAttributeValueException(ATTR_MESSAGE);
            }
            setMessage((String) attribute.getValue());
        } else {
            throw new AttributeNotFoundException(attribute.getName());
        }
    }

    @Override
    public AttributeList setAttributes(AttributeList attributes) {
        if (attributes == null || attributes.size() == 0) {
            return null;
        }
        AttributeList result = new AttributeList();
        for (Object o : attributes) {
            if (o instanceof Attribute) {
                Attribute a = (Attribute) o;
                try {
                    setAttribute(a);
                    result.add(a);
                } catch (AttributeNotFoundException e) {
                    e.printStackTrace();
                } catch (InvalidAttributeValueException e) {
                    e.printStackTrace();
                }
            }
        }
        return result;
    }
}
